# python3.7
# encoding: utf-8
"""
@author: Chenjin.Qian
@email:  chenjin.qian@xquant.com
@file:   connect_models.py
@time:   2020-06-30 15:56
"""

import os

import cx_Oracle


class ConnectOracle(object):
    def __init__(self, host=None, instance=None, user=None, password=None, port=1521):
        self.host = host if host else os.getenv("ORACLE_HOST")
        self.instance = instance if instance else os.getenv("ORACLE_INSTANCE")
        self.user = user if user else os.getenv("ORACLE_USER")
        self.pwd = password if password else os.getenv("ORACLE_PASSWORD")
        self.port = port if port != 1521 else os.getenv("ORACLE_PORT", 1521)
        self.conn = cx_Oracle.connect(f"{self.user}/{self.pwd}@{self.host}:{self.port}/{self.instance}")

    def commit(self):
        self.conn.commit()

    def ping(self):
        self.conn.ping()

    def reconnect(self):
        try:
            self.ping()
            flag = 1
        except (cx_Oracle.InterfaceError, cx_Oracle.OperationalError) as e:
            print(e)
            try:
                self.conn = cx_Oracle.connect(f"{self.user}/{self.pwd}@{self.host}:{self.port}/{self.instance}")
                flag = 1
            except (cx_Oracle.InterfaceError, cx_Oracle.OperationalError) as e:
                print(e)
                flag = 0
        finally:
            return flag

    def create_cursor(self):
        return self.conn.cursor()

    def __del__(self):
        self.conn.close()



# python3.7
# encoding: utf-8
"""
@author: Chenjin.Qian
@email:  chenjin.qian@xquant.com
@file:   __init__.py.py
@time:   2020-06-30 15:56
"""

from .connect_models import ConnectOracle
from .operate import DatabaseOperate

__version__ = "0.1.0"
__all__ = [
    "ConnectOracle",
    "DatabaseOperate"
]



# python3.7
# encoding: utf-8
"""
@author: Chenjin.Qian
@email:  chenjin.qian@xquant.com
@file:   operate.py
@time:   2020-06-30 16:29
"""

import pandas as pd


class DatabaseOperate(object):
    def __init__(self, current_conn):
        self.conn = current_conn
        self.cursor = self.conn.create_cursor()

    def check_connection(self):
        flag = self.conn.reconnect()
        return flag

    def get_all(self, sql, **kwargs):
        if not self.check_connection():
            return None
        self.cursor.execute(sql, kwargs)
        out = self.cursor.fetchall()
        return out

    def get_one(self, sql, **kwargs):
        if not self.check_connection():
            return None
        self.cursor.execute(sql, kwargs)
        out = self.cursor.fetchone()
        return out

    def one_column_list(self, sql, **kwargs):
        if not self.check_connection():
            return None
        self.cursor.execute(sql, kwargs)
        rows = self.cursor.fetchall()
        out = [i[0] for i in rows]
        return out

    def get_df(self, sql, index_name=None, **kwargs):
        if not self.check_connection():
            return None
        self.cursor.execute(sql, kwargs)
        rows = self.cursor.fetchall()
        col_name = [i[0] for i in self.cursor.description]
        data = [list(i) for i in rows]
        df = pd.DataFrame(data, columns=col_name)
        table = df if not index_name else df.set_index(index_name)
        return table

    def insert_data(self, sql, data=[]):
        if not self.check_connection():
            return None
        self.cursor.prepare(sql)
        self.cursor.execute(None, data)
        self.conn.commit()

    def execute_sql(self, sql, **kwargs):
        if not self.check_connection():
            return None
        try:
            self.cursor.execute(sql, kwargs)
            self.conn.commit()
            flag = 1
        except Exception as e:
            flag = 0
        finally:
            return flag

    def close_cursor(self):
        self.cursor.close()



# python3.7
# encoding: utf-8
"""
@author: Chenjin.Qian
@email:  chenjin.qian@xquant.com
@file:   setup.py
@time:   2020-07-01 8:42
"""

from setuptools import setup, find_packages
import oracle_connection


def read(f):
    return open(f, encoding="utf-8").read()


setup(
    name="oracle-connection",
    version=oracle_connection.__version__,
    description='A new way to operate oracle',
    long_description=read('README.rst'),
    license='MIT License',
    author="Jingxuan",
    author_email="qian_chen_jin@163.com",
    url="https://github.com/Thchoonlophon/oracle-connection",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["pandas", "cx_Oracle"],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
